@@ -33,11 +33,11 @@ class Document:
 
     def __eq__(self, other):
         if type(other) is type(self):
-            return self.name == other.name and self.path == other.path
+            return self.name == other.name and self.text == other.text
         return False
 
     def __hash__(self):
-        return hash((self.name, self.path))
+        return hash((self.name, self.text))
 
     def __repr__(self):
         return f"Document('{self.name}')"
